@@ -29,7 +29,7 @@ fn create_snapshot(
     let display_path = file.strip_prefix(display_root).unwrap();
     let display_path_str = display_path.display().to_string();
     js_runtime
-      .execute(
+      .execute_script(
         &("deno:".to_string() + &display_path_str.replace('\\', "/")),
         &std::fs::read_to_string(&file).unwrap(),
       )
@@ -68,6 +68,11 @@ fn create_compiler_snapshot(
     "deno.broadcast_channel",
     deno_broadcast_channel::get_declaration(),
   );
+  op_crate_libs.insert("deno.net", deno_net::get_declaration());
+  op_crate_libs
+    .insert("deno.net_unstable", deno_net::get_unstable_declaration());
+  op_crate_libs
+    .insert("deno.http_unstable", deno_http::get_unstable_declaration());
 
   // ensure we invalidate the build properly.
   for (_, path) in op_crate_libs.iter() {
@@ -163,6 +168,10 @@ fn create_compiler_snapshot(
       }))
     }),
   );
+  js_runtime.register_op(
+    "op_cwd",
+    op_sync(move |_state, _args: Value, _: ()| Ok(json!("cache:///"))),
+  );
   // using the same op that is used in `tsc.rs` for loading modules and reading
   // files, but a slightly different implementation at build time.
   js_runtime.register_op(
@@ -298,6 +307,18 @@ fn main() {
     "cargo:rustc-env=DENO_BROADCAST_CHANNEL_LIB_PATH={}",
     deno_broadcast_channel::get_declaration().display()
   );
+  println!(
+    "cargo:rustc-env=DENO_NET_LIB_PATH={}",
+    deno_net::get_declaration().display()
+  );
+  println!(
+    "cargo:rustc-env=DENO_NET_UNSTABLE_LIB_PATH={}",
+    deno_net::get_unstable_declaration().display()
+  );
+  println!(
+    "cargo:rustc-env=DENO_HTTP_UNSTABLE_LIB_PATH={}",
+    deno_http::get_unstable_declaration().display()
+  );
 
   println!("cargo:rustc-env=TARGET={}", env::var("TARGET").unwrap());
   println!("cargo:rustc-env=PROFILE={}", env::var("PROFILE").unwrap());
