@@ -9,7 +9,7 @@
 //!   Diagnostics are compile-time type errors, whereas JsErrors are runtime
 //!   exceptions.
 
-use crate::ast::DiagnosticBuffer;
+use crate::ast::Diagnostic;
 use crate::import_map::ImportMapError;
 use deno_core::error::AnyError;
 
@@ -17,7 +17,7 @@ fn get_import_map_error_class(_: &ImportMapError) -> &'static str {
   "URIError"
 }
 
-fn get_diagnostic_class(_: &DiagnosticBuffer) -> &'static str {
+fn get_diagnostic_class(_: &Diagnostic) -> &'static str {
   "SyntaxError"
 }
 
@@ -27,10 +27,7 @@ pub(crate) fn get_error_class_name(e: &AnyError) -> &'static str {
       e.downcast_ref::<ImportMapError>()
         .map(get_import_map_error_class)
     })
-    .or_else(|| {
-      e.downcast_ref::<DiagnosticBuffer>()
-        .map(get_diagnostic_class)
-    })
+    .or_else(|| e.downcast_ref::<Diagnostic>().map(get_diagnostic_class))
     .unwrap_or_else(|| {
       panic!(
         "Error '{}' contains boxed error of unknown type:{}",
