@@ -37,6 +37,6 @@ console.log(JSON.stringify(txt));
 
 try {
   await Deno.resolveDns("not-found-example.com", "A", nameServer);
-} catch (e) {
+} catch (_e) {
   console.log("Error thrown for not-found-example.com");
 }
