@@ -17,7 +17,7 @@ import {
 } from "./test_util.ts";
 import { stripColor } from "../../../test_util/std/fmt/colors.ts";
 
-const customInspect = Deno.customInspect;
+const customInspect = Symbol.for("Deno.customInspect");
 const {
   Console,
   cssToAnsi: cssToAnsi_,
@@ -315,25 +315,25 @@ unitTest(function consoleTestStringifyCircular(): void {
   assertEquals(
     stringify(console),
     `console {
-  log: [Function: log],
-  debug: [Function: debug],
-  info: [Function: info],
-  dir: [Function: dir],
-  dirxml: [Function: dir],
-  warn: [Function: warn],
-  error: [Function: error],
-  assert: [Function: assert],
-  count: [Function: count],
-  countReset: [Function: countReset],
-  table: [Function: table],
-  time: [Function: time],
-  timeLog: [Function: timeLog],
-  timeEnd: [Function: timeEnd],
-  group: [Function: group],
-  groupCollapsed: [Function: group],
-  groupEnd: [Function: groupEnd],
-  clear: [Function: clear],
-  trace: [Function: trace],
+  log: [Function: bound ],
+  debug: [Function: bound ],
+  info: [Function: bound ],
+  dir: [Function: bound ],
+  dirxml: [Function: bound ],
+  warn: [Function: bound ],
+  error: [Function: bound ],
+  assert: [Function: bound ],
+  count: [Function: bound ],
+  countReset: [Function: bound ],
+  table: [Function: bound ],
+  time: [Function: bound ],
+  timeLog: [Function: bound ],
+  timeEnd: [Function: bound ],
+  group: [Function: bound ],
+  groupCollapsed: [Function: bound ],
+  groupEnd: [Function: bound ],
+  clear: [Function: bound ],
+  trace: [Function: bound ],
   indentLevel: 0,
   [Symbol(isConsoleInstance)]: true
 }`,
@@ -344,7 +344,7 @@ unitTest(function consoleTestStringifyCircular(): void {
   );
   assertEquals(
     stringify({
-      [Deno.customInspect]: function () {
+      [Symbol.for("Deno.customInspect")]: function () {
         return Deno.inspect(this);
       },
     }),
@@ -879,6 +879,18 @@ unitTest(function consoleTestWithCustomInspector(): void {
   assertEquals(stringify(new A()), "b");
 });
 
+unitTest(function consoleTestWithCustomInspectorUsingInspectFunc(): void {
+  class A {
+    [customInspect](
+      inspect: (v: unknown, opts?: Deno.InspectOptions) => string,
+    ): string {
+      return "b " + inspect({ c: 1 });
+    }
+  }
+
+  assertEquals(stringify(new A()), "b { c: 1 }");
+});
+
 unitTest(function consoleTestWithCustomInspectorError(): void {
   class A {
     [customInspect](): never {
@@ -1280,8 +1292,8 @@ unitTest(function consoleTable(): void {
       `┌───────┬────────┐
 │ (idx) │ Values │
 ├───────┼────────┤
-│   a   │ "test" │
-│   b   │   1    │
+│ a     │ "test" │
+│ b     │ 1      │
 └───────┴────────┘
 `,
     );
@@ -1293,8 +1305,8 @@ unitTest(function consoleTable(): void {
       `┌───────┬────┐
 │ (idx) │ c  │
 ├───────┼────┤
-│   a   │    │
-│   b   │ 30 │
+│ a     │    │
+│ b     │ 30 │
 └───────┴────┘
 `,
     );
@@ -1304,13 +1316,13 @@ unitTest(function consoleTable(): void {
     assertEquals(
       stripColor(out.toString()),
       `┌───────┬───────┬───────┬────────┐
-│ (idx) │   0   │   1   │ Values │
+│ (idx) │ 0     │ 1     │ Values │
 ├───────┼───────┼───────┼────────┤
-│   0   │       │       │   1    │
-│   1   │       │       │   2    │
-│   2   │   3   │ [ 4 ] │        │
-│   3   │   5   │   6   │        │
-│   4   │ [ 7 ] │ [ 8 ] │        │
+│ 0     │       │       │ 1      │
+│ 1     │       │       │ 2      │
+│ 2     │ 3     │ [ 4 ] │        │
+│ 3     │ 5     │ 6     │        │
+│ 4     │ [ 7 ] │ [ 8 ] │        │
 └───────┴───────┴───────┴────────┘
 `,
     );
@@ -1322,10 +1334,10 @@ unitTest(function consoleTable(): void {
       `┌────────────┬────────┐
 │ (iter idx) │ Values │
 ├────────────┼────────┤
-│     0      │   1    │
-│     1      │   2    │
-│     2      │   3    │
-│     3      │ "test" │
+│ 0          │ 1      │
+│ 1          │ 2      │
+│ 2          │ 3      │
+│ 3          │ "test" │
 └────────────┴────────┘
 `,
     );
@@ -1342,8 +1354,8 @@ unitTest(function consoleTable(): void {
       `┌────────────┬─────┬────────┐
 │ (iter idx) │ Key │ Values │
 ├────────────┼─────┼────────┤
-│     0      │  1  │ "one"  │
-│     1      │  2  │ "two"  │
+│ 0          │ 1   │ "one"  │
+│ 1          │ 2   │ "two"  │
 └────────────┴─────┴────────┘
 `,
     );
@@ -1359,13 +1371,13 @@ unitTest(function consoleTable(): void {
     assertEquals(
       stripColor(out.toString()),
       `┌───────┬───────────┬───────────────────┬────────┐
-│ (idx) │     c     │         e         │ Values │
+│ (idx) │ c         │ e                 │ Values │
 ├───────┼───────────┼───────────────────┼────────┤
-│   a   │           │                   │  true  │
-│   b   │ { d: 10 } │ [ 1, 2, [Array] ] │        │
-│   f   │           │                   │ "test" │
-│   g   │           │                   │        │
-│   h   │           │                   │        │
+│ a     │           │                   │ true   │
+│ b     │ { d: 10 } │ [ 1, 2, [Array] ] │        │
+│ f     │           │                   │ "test" │
+│ g     │           │                   │        │
+│ h     │           │                   │        │
 └───────┴───────────┴───────────────────┴────────┘
 `,
     );
@@ -1381,13 +1393,13 @@ unitTest(function consoleTable(): void {
     assertEquals(
       stripColor(out.toString()),
       `┌───────┬────────┬──────────────────────┬────┬────────┐
-│ (idx) │   0    │          1           │ a  │ Values │
+│ (idx) │ 0      │ 1                    │ a  │ Values │
 ├───────┼────────┼──────────────────────┼────┼────────┤
-│   0   │        │                      │    │   1    │
-│   1   │        │                      │    │ "test" │
-│   2   │        │                      │    │ false  │
-│   3   │        │                      │ 10 │        │
-│   4   │ "test" │ { b: 20, c: "test" } │    │        │
+│ 0     │        │                      │    │ 1      │
+│ 1     │        │                      │    │ "test" │
+│ 2     │        │                      │    │ false  │
+│ 3     │        │                      │ 10 │        │
+│ 4     │ "test" │ { b: 20, c: "test" } │    │        │
 └───────┴────────┴──────────────────────┴────┴────────┘
 `,
     );
@@ -1447,9 +1459,9 @@ unitTest(function consoleTable(): void {
       `┌───────┬─────────┐
 │ (idx) │ Values  │
 ├───────┼─────────┤
-│   0   │ "Hello" │
-│   1   │ "你好"  │
-│   2   │ "Amapá" │
+│ 0     │ "Hello" │
+│ 1     │ "你好"  │
+│ 2     │ "Amapá" │
 └───────┴─────────┘
 `,
     );
@@ -1464,8 +1476,8 @@ unitTest(function consoleTable(): void {
       `┌───────┬───┬───┐
 │ (idx) │ 0 │ 1 │
 ├───────┼───┼───┤
-│   0   │ 1 │ 2 │
-│   1   │ 3 │ 4 │
+│ 0     │ 1 │ 2 │
+│ 1     │ 3 │ 4 │
 └───────┴───┴───┘
 `,
     );
@@ -1477,9 +1489,9 @@ unitTest(function consoleTable(): void {
       `┌───────┬───┐
 │ (idx) │ b │
 ├───────┼───┤
-│   1   │ 5 │
-│   2   │   │
-│   3   │ 6 │
+│ 1     │ 5 │
+│ 2     │   │
+│ 3     │ 6 │
 └───────┴───┘
 `,
     );
@@ -1491,10 +1503,10 @@ unitTest(function consoleTable(): void {
       `┌───────┬───┬───┐
 │ (idx) │ a │ b │
 ├───────┼───┼───┤
-│   0   │ 0 │   │
-│   1   │ 1 │ 1 │
-│   2   │ 2 │   │
-│   3   │ 3 │ 3 │
+│ 0     │ 0 │   │
+│ 1     │ 1 │ 1 │
+│ 2     │ 2 │   │
+│ 3     │ 3 │ 3 │
 └───────┴───┴───┘
 `,
     );
@@ -1509,10 +1521,10 @@ unitTest(function consoleTable(): void {
       `┌───────┬───┬───┬───┐
 │ (idx) │ a │ b │ c │
 ├───────┼───┼───┼───┤
-│   0   │ 0 │   │   │
-│   1   │ 1 │   │ 1 │
-│   2   │ 2 │   │   │
-│   3   │ 3 │   │ 3 │
+│ 0     │ 0 │   │   │
+│ 1     │ 1 │   │ 1 │
+│ 2     │ 2 │   │   │
+│ 3     │ 3 │   │ 3 │
 └───────┴───┴───┴───┘
 `,
     );
