@@ -3,6 +3,7 @@ import {
   assert,
   assertEquals,
   assertThrowsAsync,
+  deferred,
   fail,
   unimplemented,
   unitTest,
@@ -820,7 +821,9 @@ unitTest(function responseRedirect(): void {
 unitTest(async function responseWithoutBody(): Promise<void> {
   const response = new Response();
   assertEquals(await response.arrayBuffer(), new ArrayBuffer(0));
-  assertEquals(await response.blob(), new Blob([]));
+  const blob = await response.blob();
+  assertEquals(blob.size, 0);
+  assertEquals(await blob.arrayBuffer(), new ArrayBuffer(0));
   assertEquals(await response.text(), "");
   await assertThrowsAsync(async () => {
     await response.json();
@@ -1195,3 +1198,34 @@ unitTest(
     assertEquals(response.headers.get("Host"), addr);
   },
 );
+
+unitTest(
+  { perms: { net: true } },
+  async function fetchNoServerReadableStreamBody() {
+    const done = deferred();
+    const body = new ReadableStream({
+      start(controller) {
+        controller.enqueue(new Uint8Array([1]));
+        setTimeout(() => {
+          controller.enqueue(new Uint8Array([2]));
+          done.resolve();
+        }, 1000);
+      },
+    });
+    const nonExistantHostname = "http://localhost:47582";
+    await assertThrowsAsync(async () => {
+      await fetch(nonExistantHostname, { body, method: "POST" });
+    }, TypeError);
+    await done;
+  },
+);
+
+unitTest(
+  { perms: { net: true } },
+  async function fetchHeadRespBody() {
+    const res = await fetch("http://localhost:4545/echo_server", {
+      method: "HEAD",
+    });
+    assertEquals(res.body, null);
+  },
+);
