@@ -9,6 +9,7 @@ import {
   assertThrowsAsync,
   deferred,
   delay,
+  fail,
   unitTest,
 } from "./test_util.ts";
 
@@ -544,6 +545,49 @@ unitTest(
   },
 );
 
+unitTest({ perms: { net: true } }, async function httpRequestLatin1Headers() {
+  const promise = (async () => {
+    const listener = Deno.listen({ port: 4501 });
+    for await (const conn of listener) {
+      const httpConn = Deno.serveHttp(conn);
+      for await (const { request, respondWith } of httpConn) {
+        assertEquals(request.headers.get("X-Header-Test"), "á");
+        await respondWith(
+          new Response("", { headers: { "X-Header-Test": "Æ" } }),
+        );
+        httpConn.close();
+      }
+      break;
+    }
+  })();
+
+  const clientConn = await Deno.connect({ port: 4501 });
+  const requestText =
+    "GET / HTTP/1.1\r\nHost: 127.0.0.1:4501\r\nX-Header-Test: á\r\n\r\n";
+  const requestBytes = new Uint8Array(requestText.length);
+  for (let i = 0; i < requestText.length; i++) {
+    requestBytes[i] = requestText.charCodeAt(i);
+  }
+  let written = 0;
+  while (written < requestBytes.byteLength) {
+    written += await clientConn.write(requestBytes.slice(written));
+  }
+
+  let responseText = "";
+  const buf = new Uint8Array(1024);
+  let read;
+  while ((read = await clientConn.read(buf)) !== null) {
+    for (let i = 0; i < read; i++) {
+      responseText += String.fromCharCode(buf[i]);
+    }
+  }
+  clientConn.close();
+
+  assert(/\r\n[Xx]-[Hh]eader-[Tt]est: Æ\r\n/.test(responseText));
+
+  await promise;
+});
+
 unitTest(
   { perms: { net: true } },
   async function httpServerRequestWithoutPath() {
@@ -588,3 +632,60 @@ unitTest(
     await promise;
   },
 );
+
+unitTest({ perms: { net: true } }, async function httpServerWebSocket() {
+  const promise = (async () => {
+    const listener = Deno.listen({ port: 4501 });
+    for await (const conn of listener) {
+      const httpConn = Deno.serveHttp(conn);
+      const { request, respondWith } = (await httpConn.nextRequest())!;
+      const {
+        response,
+        websocket,
+      } = await Deno.upgradeWebSocket(request);
+      websocket.onerror = () => fail();
+      websocket.onmessage = (m) => {
+        websocket.send(m.data);
+        websocket.close();
+      };
+      await respondWith(response);
+      break;
+    }
+  })();
+
+  const def = deferred();
+  const ws = new WebSocket("ws://localhost:4501");
+  ws.onmessage = (m) => assertEquals(m.data, "foo");
+  ws.onerror = () => fail();
+  ws.onclose = () => def.resolve();
+  ws.onopen = () => ws.send("foo");
+  await def;
+  await promise;
+});
+
+unitTest({ perms: { net: true } }, async function httpCookieConcatenation() {
+  const promise = (async () => {
+    const listener = Deno.listen({ port: 4501 });
+    for await (const conn of listener) {
+      const httpConn = Deno.serveHttp(conn);
+      for await (const { request, respondWith } of httpConn) {
+        assertEquals(new URL(request.url).href, "http://127.0.0.1:4501/");
+        assertEquals(await request.text(), "");
+        assertEquals(request.headers.get("cookie"), "foo=bar; bar=foo");
+        respondWith(new Response("ok"));
+      }
+      break;
+    }
+  })();
+
+  const resp = await fetch("http://127.0.0.1:4501/", {
+    headers: [
+      ["connection", "close"],
+      ["cookie", "foo=bar"],
+      ["cookie", "bar=foo"],
+    ],
+  });
+  const text = await resp.text();
+  assertEquals(text, "ok");
+  await promise;
+});
